/**
 * Build the vertices of a sphere. The horizontal divisions can be limited to build parts of spheres (e.g. half-sphere)
 * 
 * @param {Number} radius 
 * @param {Number} vDivisions 
 * @param {Number} hDivisions 
 * @param {Number} hStart 
 * @param {Number} hEnd 
 */
export function buildSphereVertices(radius, vDivisions, hDivisions, hStart, hEnd) {
    const vertices = [];
    const verticalStep = 2 * Math.PI / vDivisions;
    const horizontalStep = Math.PI / hDivisions;
    // loop over all vertical divisions (parallels)
    for (let i = 0; i < vDivisions; i++) {
        const theta = i * verticalStep;
        // loop over all meridians (except for the first and last step, to reduce vertex count)
        for (let j = 1 + hStart; j < hEnd; j++) {
            const phi = j * horizontalStep;
            
            // convert sphere coordinates to cartesian
            const x = radius * Math.sin(phi) * Math.sin(theta);
            const y = radius * Math.cos(phi);
            const z = radius * Math.sin(phi) * Math.cos(theta);
            vertices.push(x, y, z);
        }
    }

    // add top and bottom vertex
    if (hStart == 0) {
        vertices.push(0, radius, 0);
    }
    if (hEnd == hDivisions) {
        vertices.push(0, -radius, 0);
    }

    return vertices;
}

/**
 * Build the tri indices for a (part) sphere
 * 
 * @param {*} sphereVertexCount 
 * @param {*} vDivisions 
 * @param {*} hDivisions 
 * @param {*} hStart 
 * @param {*} hEnd 
 */
export function buildSphereIndices(sphereVertexCount, vDivisions, hDivisions, hStart, hEnd) {
    const indices = [];

    // set up the index array for the tris
    // our actual number horizontal divisions (since we skip it at 0 if hStart == 0)
    let actualHorizontalDivisions = hEnd - hStart;
    if (hStart == 0)
        actualHorizontalDivisions--;
    
    for (let i = 0; i < vDivisions; i++) {
        // if we have a top vertex and / or a bottom vertex
        // connect the upper and outer edge to the top and bottom center points
        if (hStart == 0) {
            indices.push(
                // top vertex
                sphereVertexCount,
                i * actualHorizontalDivisions,
                ((i + 1) * actualHorizontalDivisions) % sphereVertexCount
            );
        }
        if (hEnd == hDivisions) {
            indices.push(
                // bottom vertex
                sphereVertexCount + 1,
                (i + 1) * actualHorizontalDivisions - 1,
                (((i + 2) * actualHorizontalDivisions) - 1) % sphereVertexCount
            )
        }
        // build the tris for the rest of the shell slice
        for (let j = hStart; j < actualHorizontalDivisions - 1; j++) {
            const cit = i * actualHorizontalDivisions + j;
            const cib = i * actualHorizontalDivisions + j + 1;
            const nit = ((i + 1) * actualHorizontalDivisions + j) % sphereVertexCount;
            const nib = ((i + 1) * actualHorizontalDivisions + j + 1) % sphereVertexCount;

            indices.push(
                cit, cib, nit,
                nit, cib, nib
            );
        }
    }
    return indices;
}